//#ifndef __CINT__
#include "RooGlobalFunc.h"
//#endif
#include "RooRealVar.h"
#include "RooArgList.h"
#include "RooFormulaVar.h"
#include "RooDataSet.h"
#include "RooGaussian.h"
#include "RooChebychev.h"
#include "RooExponential.h"
#include "RooBifurGauss.h"
#include "RooAddModel.h"
#include "RooProdPdf.h"
#include "TCanvas.h"
#include "RooPlot.h"
#include "RooHist.h"
#include "RooCBShape.h"
#include "RooPolynomial.h"
#include "RooBinning.h"
#include "TH1.h"
#include "TH2.h"
#include "RooAddPdf.h"
#include "RooProdPdf.h"
#include "RooFitResult.h"
#include "RooGenericPdf.h"
#include "RooLandau.h"
#include "TChain.h"
#include "TCanvas.h"
#include "TAxis.h"
#include "RooPlot.h"
#include "RooCategory.h"
#include "RooSuperCategory.h"
#include "RooSimultaneous.h"
#include "RooNLLVar.h"
#include "TFile.h"
//#include "RooJohnson"
#include "RooFit.h"


using namespace RooFit ;
using namespace std;


// const int    belle2_font = 133;
// const double belle2_tsize = 26;


void unbinned_2D_fit_Kpipi0_RS_pi0M_conv()  
{
	

  TFile *f = new TFile("DtoKpipi0_RS_mc14a_100fb_total_light_2012_minus_with_cut_2D_tree_new_criteria.root");

  TTree *tree = (TTree*)f->Get("distributions");
  cout << "Entries = " << tree->GetEntries() << endl;
	
  
  //create both fit observables and get the data in RooFit land
  RooRealVar pi0_M_prefit("pi0_m_prefit","#it{m}(#it{#pi}^{0})", 0.1, 0.18, "GeV/#it{c}^{2}");
  RooRealVar pi0_ErrM("pi0_errm","#it{m}(#it{#pi}^{0} uncertainty)", 0, 0.03, "GeV/#it{c}^{2}");
  RooDataSet* dataxy = new RooDataSet("data","data",tree,RooArgSet(pi0_M_prefit, pi0_ErrM));

  
 
  // Create Double Gaussian resolution model where the sigma are replaced by s_1*pi0_ErrM and s_2*pi0_ErrM
  RooRealVar f_G1("f_{G1}", "frac_gauss1",0.785);
  RooRealVar mu_G1("#mu_{G1}", "mean_gauss1",2.0103010);
  RooRealVar mu_G2("#mu_{G2}", "mean_gauss2",2.0103010);
  RooRealVar sigma_G1("#sigma_{G1}", "sigma_gauss1",0.000926);
  RooRealVar sigma_G2("#sigma_{G2}", "sigma_gauss2",0.0159);
  RooGaussian gauss1("gauss1_sig", "1st gaussian PDF", pi0_ErrM, mu_G1, sigma_G1);
  RooGaussian gauss2("gauss2_sig", "2nd gaussian PDF", pi0_ErrM, mu_G2, sigma_G2);
  RooAddPdf doublegaussian("doublegaussian_sig", "doublegaussian_sig", gauss1, gauss2, f_G1);
  
 
  RooTruthModel deltafun("truthm","truth model",pi0_M_prefit) ;

  // Construct landau (x) gauss
  RooFFTConvPdf conv("conv","delta (X) doublegaussian",pi0_M_prefit, deltafun, doublegaussian) ;
 
 
  // Parameters for johnson function
  RooRealVar mu("mu", "mu", 0.04, 0.0, 10.0);
  RooRealVar sigma("sigma", "sigma", 0.005, 0.0, 10.0);
  RooRealVar gamma("gamma", "gamma", -1.5, -5.0, 5.0);
  RooRealVar delta("delta", "delta", 1.0, 0.0, 10.0);

  RooJohnson johnson("johnson", "Johnson PDF", pi0_M_prefit, mu, sigma, gamma, delta);

  RooProdPdf pdf("pdf", "conv*johnson", conv, johnson);

  //perform the fit
  RooFitResult *fitresult = pdf.fitTo(*dataxy, Save(true), Strategy(2), Extended(true));


  //fit
  //RooFitResult *fitresult = pdf.fitTo(data, Save(true), Strategy(2), Extended(true));
  
	/*-----------------------------------PLOT THE RESULTS-------------------------------------*/
  //Plot pi0_M_prefit projections
  RooPlot *xframe = pi0_M_prefit.frame();
  dataxy->plotOn(xframe);
  pdf.plotOn(xframe);
  pdf.paramOn(xframe,dataxy);
  

  //Plot pi0_ErrM projections
  RooPlot *yframe = pi0_ErrM.frame();
  dataxy->plotOn(yframe);
  pdf.plotOn(yframe);
  

  // Make canvas and draw RooPlots
  TCanvas *c = new TCanvas("c", "c", 1200, 600);
  c->Divide(2);
  c->cd(1);
  gPad->SetLeftMargin(0.15);
  xframe->GetYaxis()->SetTitleOffset(1.6);
  xframe->Draw();
  c->cd(2);
  gPad->SetLeftMargin(0.15);
  yframe->GetYaxis()->SetTitleOffset(1.6);
  yframe->Draw();

}
